(function() {
    var hasOwnProperty = Object.prototype.hasOwnProperty;
    var propIsEnumerable = Object.prototype.propertyIsEnumerable;
    var isIOS = navigator.userAgent.indexOf("iPhone") > 0;
    /**
     * 模拟ES6的assign方法（合并JSON对象）
     * @type {{isObj: isObj, toObject: toObject, assignKey: assignKey, assign: assign, deepAssign: deepAssign}}
     */
    var assignUtils = {
        isObj: function(x) {
            var type = typeof x;
            return x !== null && (type === 'object' || type === 'function');
        },
        toObject: function(val) {
            if (val === null || val === undefined) {
                throw new TypeError('Cannot convert undefined or null to object');
            }

            return Object(val);
        },
        assignKey: function(to, from, key) {
            var val = from[key];

            if (val === undefined || val === null) {
                return;
            }

            if (hasOwnProperty.call(to, key)) {
                if (to[key] === undefined || to[key] === null) {
                    throw new TypeError('Cannot convert undefined or null to object (' + key + ')');
                }
            }

            if (!hasOwnProperty.call(to, key) || !this.isObj(val)) {
                to[key] = val;
            } else {
                to[key] = this.assign(Object(to[key]), from[key]);
            }
        },
        assign: function(to, from) {
            if (to === from) {
                return to;
            }

            from = Object(from);

            for (var key in from) {
                if (hasOwnProperty.call(from, key)) {
                    this.assignKey(to, from, key);
                }
            }

            if (Object.getOwnPropertySymbols) {
                var symbols = Object.getOwnPropertySymbols(from);

                for (var i = 0; i < symbols.length; i++) {
                    if (propIsEnumerable.call(from, symbols[i])) {
                        this.assignKey(to, from, symbols[i]);
                    }
                }
            }

            return to;
        },
        deepAssign: function(target) {
            target = this.toObject(target);

            for (var s = 1; s < arguments.length; s++) {
                this.assign(target, arguments[s]);
            }

            return target;
        }
    };
    var _bindCallback = function(scope, fn1, fn2) {
        scope.success = fn1 ? fn1 : function() {};
        scope.failure = fn2 ? fn2 : function() {};
    };

    // function _validateAttr(attr) {
    //     return attr != null && typeof(attr) != "undefined"; //这个逻辑有问题，若undefined会报错，需要反过来。
    // }

    function isDefined(attr) {
        return typeof attr !== 'undefined' && attr != null;
    }

    if (!window.js2native) {
        window.js2native = {
            exec: function(className, methodName, params, invoker) {
                var arr = params;
                var arrStr = '';
                if (arr && arr.length > 0) {
                    for (var i = 0; i < arr.length; i++) {
                        arrStr += '$';
                        arrStr += arr[i];
                    }
                    arrStr = arrStr.substring(1);
                }
                var _callbackIdVal = "_callback_" + new Date().getTime() + randomString(5);
                eval("window." + _callbackIdVal + " = {}; window." + _callbackIdVal + ".success = invoker.success; if(invoker.failure){window." + _callbackIdVal + ".failure = invoker.failure;}else{window." + _callbackIdVal + ".failure=function(){}}");
                var url = "callbackId=" + _callbackIdVal + "&className=" + className + "&method=" + methodName + "&params=" + encodeURIComponent(arrStr);
                if (isIOS) {
                    url = "https://callfunction//" + url;
                } else {
                    url = "callfunction://" + url;
                }
                url += ("&currentPage=" + pageName());
                url += ("&tt=" + new Date().getTime());
                createFrame(url);
            }
        };
    }

    function  randomString(len)  {
        var result = "_";
        for(var i = 0; i < len; i++)  {
            result += Math.ceil(Math.random() * 100);
        }
        return result;
    }

    function pageName() {
        var strUrl = location.href;
        var arrUrl = strUrl.split("/");
        var strPage = arrUrl[arrUrl.length - 1];
        return strPage;
    }

    function createFrame(url) {
        var iframe = document.createElement("iframe");
        iframe.src = url;
        iframe.style.display = 'none';
        document.body.appendChild(iframe);
        iframe.parentNode.removeChild(iframe);
        iframe = null;
    }

    var plugins = {
        //**********************************  InfoPlugin  ************************************//
        /**
         * @class plugins.InfoPlugin
         * 信息获取插件
         * @version:
         * @description: 信息获取插件
         */
        InfoPlugin: {
            /**
             * 获取上下文信息
             */
            // getCtx: {
            //     action: function (param) {
            //         if (typeof(param) === "object") {
            //             _bindCallback(this, param.success, param.error);
            //             js2native.exec("InfoPlugin", "getCtx", [],this);
            //         }
            //     }
            // },
            getCtx: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("InfoPlugin", "getCtx", [], this);
                }
            },
            getVersion: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("InfoPlugin", "getVersion", [], this);
                }
            },
            getDeviceIdentifier: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("InfoPlugin", "getDeviceIdentifier", [], this);
                }
            },
            getOSInfo: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("InfoPlugin", "getOSInfo", [], this);
                }
            },
            getRegistrationId: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("InfoPlugin", "getRegistrationId", [], this);
                }
            },

            /**
             * @method getCoordinate
             * 获取当前经纬度
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            getCoordinate: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("InfoPlugin", "getCoordinate", [], this);
                }
            },
            /**
             * @method getGeoLocation
             * 获取当前定位省，市，区名字
             * @param {String} precision ("city"--获取城市名,"province"--获取省份名, "district"--获取地区名)
             * @param {Function} param.success 成功回调函数
             *                   如果precision = "province"||"district",则直接返省份名称或地区名称，
             *                   如果precision = "city",则返回一个map：{"cityName":"城市名","cityCode":"行政区划代码","longitude":"经度","latitude":"纬度"}
             * @param {Function} param.error 失败回调函数
             */
            getGeoLocation: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    var precision = !isDefined(param.precision) ? "city" : param.precision;
                    js2native.exec("InfoPlugin", "getGeoLocation", [precision], this);
                }
            }
        },
        //**********************************  TabBarPlugin  ************************************//
        /**
         * @class plugins.TabBarPlugin
         */
        TabBarPlugin: {
            hideTabBar: function() {
                js2native.exec("TabBarPlugin", "hideTabBar", [], this);
            },
            showTabBar: function() {
                js2native.exec("TabBarPlugin", "showTabBar", [], this);
            },
            selectTabBarIndex: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    index = !isDefined(param.index) ? 0 : param.index;
                    if (index < 0) {
                        index = 0;
                    }
                    js2native.exec("TabBarPlugin", "selectTabBarIndex", [index], this);
                }
            }
        },
        /**
         * @class plugins.NumberKeyBoardPlugin
         * 0000 | 空成功回调
         * 2000 | 输入达到最大长度
         * 2100 | 用户手动关闭键盘
         *
         * 9988 | 传参错误（参数数量/类型/格式等参数有错误）
         * 4100 | 正则表达式不匹配
         * 4200 | 弱密码校验不通过
         * 4300 | 输入内容为空
         * 4400 | 加密参数有误
         * 4500 | 加密失败
         * 4600 | 密码两次输入不一致（密码确认失败）
         */
        NumberKeyBoardPlugin: {

            /**
             * @method show
             * @alias inputPwdForVerify
             * 首次输入密码
             * 成功回调：0000, 2000, 2100
             * 失败回调：9988, 4200, 4300
             *
             * @param {Object} param
             * @param {Number} [param.maxLength] 最大长度
             * @param {String} [param.pwdKey] 用作标识位
             * @param {String} [param.random = "false"] 是否随机键位
             * @param {String} [param.visualResponse = "true"] 按键是否有视觉反馈
             * @param {String} [param.offset] 输入框相对有页面的高度，用于客户端适配输入框位置（防止被键盘挡住）
             * @param {String} [param.random = "false"] 是否随机键位
             * @param {String} [param.objId] 输入框id
             * @param {String} [param.visualResponse = "true"] 按键是否有视觉反馈
             * @param {String} [param.checkWeakPwd = "false"] 是否要做弱密码校验
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            show: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.pwdKey)) {
                    param.pwdKey = "defaultkey";
                }
                if (!isDefined(param.random)) {
                    param.random = "0";
                } else {
                    if (param.random === "true" || param.random === true) {
                        param.random = "1";
                    } else if (param.random === "false" || param.random === false) {
                        param.random = "0";
                    }
                }
                if (!isDefined(param.maxLength)) {
                    param.maxLength = 6;
                }
                if (!isDefined(param.visualResponse)) {
                    param.visualResponse = "true";
                }
                if (!isDefined(param.checkWeakPwd)) {
                    param.checkWeakPwd = "true";
                }
                js2native.exec("NumberKeyBoardPlugin", "inputPwdForVerify", [JSON.stringify(param)], this);
            },
            /**
             * @method verifyPwd
             * 验证输入框标识为pwdKey1和pwdkey2的密码是否一致，如果一致，通过加密后，返回加密后的数据。
             * 可以获得单一密码的密文（key1和key2传同样参数）
             * @param {Object} data
             * @param {String} data.pwdKey1 [] 第一个密码的标识
             * @param {String} data.pwdKey2 [] 第二个密码的标识
             * @param {Object} data.algorithm [] 算法描述
             * @param {String} data.algorithm.algName 算法名（"RSA"/"SHA1"）
             * @param {String} data.algorithm.pk 加密使用的秘钥(算法不需要秘钥时可不传)
             * @param {String} data.algorithm.pinType 加密拼接方式,默认为0
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            verifyPwd: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.pwdKey1)) {
                        throw "pwd key 1 can not be empty";
                    }
                    if (!isDefined(param.pwdKey2)) {
                        throw "pwd key 2 can not be empty";
                    }
                } catch (_ex) {
                    if (isDefined(param.error)) {
                        param.error(_ex);
                    }
                    return;
                }
                if (!isDefined(param.algorithm)) {
                    param.algorithm = {
                        algName: "SHA1"
                    }
                }
                js2native.exec("NumberKeyBoardPlugin", "verifyPwd", [JSON.stringify(param)], this);
            },
            /**
             * @method getNumStr
             * 调起键盘进行输入
             * @param {Object} data
             * @param {String} [data.val = ""] 输入框当前值
             * @param {String} [data.maxLength = ""] 可输入的最大长度
             * @param {String} data.offset [] 输入框相对有页面的高度，用于客户端适配输入框位置（防止被键盘挡住）
             * @param {String} [data.random = "false"] 是否随机键位
             * @param {String} data.objId [] 输入框id
             * @param {String} [data.visualResponse = "true"] 按键是否有视觉反馈
             * @param {String} data.regex [] 正则表达式，可空。不为空时验证正则。
             * @param {String} data.labelText [] 文本水印
             * @param {String} [data.maxAmount = ""] 可输入的数字的最大值
             * @param {String} [data.minAmount = "0"] 可输入的数字的最小值
             * @param {String} [data.scrollBar = "false"] 数字键盘是否带滚轮
             * @param {String} [data.step = "100"] 滚轮键盘的步数（刻度间隔大小）
             * @param {String} [data.type = ""] 数字键盘、身份证键盘、密码键盘的区分字段 "."为数字键盘带小数点，""(空串)/"N"为数字键盘不带小数点。"x"为身份证键盘,"P"为密码键盘（需要加密，向前端返回*）。
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            //Todo: 确认该方法的实际用途，调起键盘输入？？
            getNumStr: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.scrollBar)) {
                        param.scrollBar = "false";
                    }
                    if (!isDefined(param.minAmount)) {
                        param.minAmount = "0";
                    }
                    if (!isDefined(param.maxAmount)) {
                        param.maxAmount = "";
                    }
                    if (!isDefined(param.step)) {
                        param.step = "100";
                    }
                    if (!isDefined(param.random)) {
                        param.random = "0";
                    } else {
                        if (param.random === "true" || param.random === true) {
                            param.random = "1";
                        } else if (param.random === "false" || param.random === false) {
                            param.random = "0";
                        }
                    }
                    if (!isDefined(param.algorithm)) {
                        param.algorithm = {
                            algName: "SHA1"
                        }
                    }
                    if (!isDefined(param.visualResponse)) {
                        param.visualResponse = "true";
                    }
                    if (!isDefined(param.regex)) {
                        param.regex = "";
                    } else if (param.regex.indexOf("$") > -1) {
                        param.regex = param.regex.replace("$", "_d011Ar_");
                    }
                    js2native.exec("NumberKeyBoardPlugin", "getNumStr", [JSON.stringify(param)], this);
                }
            },
            /**
             * @method clean
             * 清空存储的所有pwd
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            clean: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("NumberKeyBoardPlugin", "clean", [JSON.stringify(param)], this);
                }
            },
            /**
             * @method hide
             * 隐藏键盘
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            hide: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("NumberKeyBoardPlugin", "hide", [], this);
                }
            }
        },
        /**
         * @class plugins.KeyBoardPlugin
         * 全键盘插件
         */
        KeyBoardPlugin: {
            /**
             * @method show
             * 弹出密码全键盘，输入明文密码不加密
             * 成功回调：0000, 2000, 2100
             * 失败回调：9988, 4200, 4300
             *
             * @param {Object} param
             * @param {Number} [param.maxLength] 最大长度
             * @param {String} [param.pwdKey] 用作标识位
             * @param {String} [param.random = "false"] 是否随机键位
             * @param {String} [param.visualResponse = "true"] 按键是否有视觉反馈
             * @param {String} [param.offset] 输入框相对有页面的高度，用于客户端适配输入框位置（防止被键盘挡住）
             * @param {String} [param.random = "false"] 是否随机键位
             * @param {String} [param.objId] 输入框id
             * @param {String} [data.visualResponse = "true"] 按键是否有视觉反馈
             * @param {String} [param.regex] 正则表达式，可空。不为空时验证正则。
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            show: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.pwdKey)) {
                    param.pwdKey = "defaultkey";
                }
                if (!isDefined(param.random)) {
                    param.random = "0";
                } else {
                    if (param.random === "true" || param.random === true) {
                        param.random = "2";
                    } else if (param.random === "false" || param.random === false) {
                        param.random = "0";
                    }
                }
                if (!isDefined(param.maxLength)) {
                    param.maxLength = 16;
                }
                if (!isDefined(param.regex)) {
                    param.regex = "";
                } else if (param.regex.indexOf("$") > -1) {
                    param.regex = param.regex.replace("$", "_d011Ar_");
                }
                if (!isDefined(param.visualResponse)) {
                    param.visualResponse = "true";
                }
                js2native.exec("KeyBoardPlugin", "inputPwdForVerify", [JSON.stringify(param)], this);
            },
            /**
             * @method verifyPwd
             * 比较两次密码输入是否相同 (校验并加密）
             * 不可单独调用，需要使用到调用show（inputPwdFroVerify）之后保存的key1
             * 第二次输入密码，根据pwdKey和第一次的密码进行比对。密码一致则返回加密后的密码。
             * 成功回调：返回加密后的密码。
             * 失败回调：9988, 4200, 4300, 4400, 4500, 4600。
             *
             * @param {Object} param
             * @param {Object} [param.algorithm] 算法描述
             * @param {String} param.algorithm.algName 算法名（"RSA"/"SHA1"）
             * @param {String} param.algorithm.pk 加密使用的秘钥(算法不需要秘钥时可不传)
             * @param {String} param.algorithm.pinType 加密拼接方式,默认为0
             * @param {String} [param.offset] 输入框相对有页面的高度，用于客户端适配输入框位置（防止被键盘挡住）
             * @param {String} [param.random = "false"] 是否随机键位
             * @param {String} [param.objId] 输入框id
             * @param {String} [param.visualResponse = "true"] 按键是否有视觉反馈
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            verifyPwd: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.pwdKey1)) {
                        throw "pwd key 1 can not be empty";
                    }
                    if (!isDefined(param.pwdKey2)) {
                        throw "pwd key 2 can not be empty";
                    }
                } catch (_ex) {
                    if (isDefined(param.error)) {
                        param.error(_ex);
                    }
                    return;
                }
                if (!isDefined(param.algorithm)) {
                    param.algorithm = {
                        algName: "SHA1"
                    }
                }
                js2native.exec("KeyBoardPlugin", "verifyPwd", [JSON.stringify(param)], this);
            },
            /**
             * Todo: 需确认该方法用途，获取已加密的密文？？
             */
            getPwd: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.random)) {
                        param.random = "0";
                    } else {
                        if (param.random === "true" || param.random === true) {
                            param.random = "2";
                        } else if (param.random === "false" || param.random === false) {
                            param.random = "0";
                        }
                    }
                    if (!isDefined(param.algorithm)) {
                        param.algorithm = {
                            algName: "SHA1"
                        }
                    }
                    if (!isDefined(param.visualResponse)) {
                        param.visualResponse = "true";
                    }
                    js2native.exec("KeyBoardPlugin", "getPwd", [JSON.stringify(param)], this);
                }
            },

            /**
             * @method validatePwd
             * 验证密码是否满足正则
             * @param {Object} param
             * @param {String} param.pwdKey pwdKey
             * @param {String} param.regex 正则表达式
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            validatePwd: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.regex)) {
                        param.regex = "";
                    } else if (param.regex.indexOf("$") > -1) {
                        param.regex = param.regex.replace("$", "_d011Ar_");
                    }
                    js2native.exec("KeyBoardPlugin", "validatePwd", [JSON.stringify(param)], this);
                }
            },
            /**
             * @method clean
             * 清空存储的所有pwd
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            clean: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("KeyBoardPlugin", "clean", [JSON.stringify(param)], this);
                }
            },
            /**
             * @method hide
             * 隐藏键盘
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            hide: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("KeyBoardPlugin", "hide", [], this);
                }
            }
        },
        /**
         * @class plugins.MaskPlugin
         * 页面遮罩插件
         */
        MaskPlugin: {
            /**
             * @method showMask
             * 显示遮罩
             * @param {Object} param
             * @param {String} [param.maskArea = 'web'] 遮罩区域 （web web页面 full 全屏 head 头部）
             * @param {String} [param.timeout = '20'] 超时时间
             * @param {String} [param.label = '加载中...'] 显示的文字
             * @param {String} [param.style = 'default'] 样式
             * @param {String} [param.color = '#70000000'] 遮罩背景颜色
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            showMask: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    var maskArea = !isDefined(param.maskArea) ? "web" : param.maskArea;
                    var timeout = !isDefined(param.timeout) ? "20" : param.timeout;
                    var label = !isDefined(param.label) ? "加载中..." : param.label;
                    var style = !isDefined(param.style) ? "default" : param.style;
                    var color = !isDefined(param.color) ? "#70000000" : param.color;
                    js2native.exec("MaskPlugin", "showMask", [style, maskArea, timeout, label, color], this);
                }
            },
            /**
             * @method unMask
             * 隐藏遮罩
             * @param 无
             */
            unMask: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                }
                js2native.exec("MaskPlugin", "unMask", [], this);
            }
        },
        //**********************************  GestureLockerPlugin  ************************************//
        /**
         * @class plugins.GestureLockerPlugin
         * 手势密码组件
         */
        GestureLockerPlugin: {
            /**
             * @method set
             * 设置当前用户手势密码
             * @param {Object} param
             * @param {String} [param.style = "default"] 设置手势密码页的样式
             * @param {String} [param.title] 设置手势页的顶部title
             * @param {Boolean} [param.showBack] 是否有返回按钮
             * @param {Object} [param.tips] 右上按钮的属性（用作取消本次设置）
             * @param {String} [param.tips.tipStr] 按钮文字
             * @param {String} [param.tips.cancelNotice] 客户端alert提示文字
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            set: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.style)) {
                        param.style = "default";
                    }
                    if (!isDefined(param.title)) {
                        param.title = "设置手势密码";
                    }
                    if (!isDefined(param.showBack)) {
                        param.showBack = false;
                    }
                    if (!isDefined(param.tips)) {
                        param.tips = {
                            tipStr: "下次设置",
                            cancelNotice: "下次请到 我的->设置 中设置"
                        }
                    }
                    js2native.exec("GestureLockerPlugin", "set", [JSON.stringify(param)], this);
                }
            },
            /**
             * @method cancel
             * 取消当前用户手势
             */
            cancel: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("GestureLockerPlugin", "cancel", [], this);
                }
            },
            /** 
             * @method modify
             * 修改当前用户手势
             * @param {String} authMode 验证方式 "gesture"-使用手势验证（默认值） "loginPwd"-使用登录密码验证
             */
            modify: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.authMode)) {
                        param.authMode = "gesture";
                    }
                    js2native.exec("GestureLockerPlugin", "modify", [JSON.stringify(param)], this);
                }
            }
        },
        /**
         * @class plugins.UserInfoPlugin
         * 用户信息插件
         */
        UserInfoPlugin: {
            /**
             * @method setAccessToken
             * 设置user的accessToken
             * @param {String} userId 用户id
             * @param {String} accessToken
             * @param {String} [store = "0"] 是否存储在文件中 0-不存文件 1-存文件
             */
            setAccessToken: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.userID)) throw "userID不能为空";
                        if (!isDefined(param.token)) throw "token不能为空";
                        var store = !isDefined(param.store) ? "0" : param.store;
                        js2native.exec("UserInfoPlugin", "setAccessToken", [param.userID, param.token, store], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            /**
             * @method setRefreshToken
             * 设置user的refreshToken
             * @param {Object} param
             * @param {String} param.userId 用户id
             * @param {String} param.refreshToken
             * @param {Number} param.accessTokenLiveTime refreshToken的存活时间
             * @param {String} [store = "0"] 是否存储在文件中 0-不存文件 1-存文件
             */
            setRefreshToken: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.userID)) throw "userID不能为空";
                        if (!isDefined(param.refreshToken)) throw "token不能为空";
                        var store = !isDefined(param.store) ? "0" : param.store;
                        var accessTokenLiveTime = !isDefined(param.accessTokenLiveTime) ? 15 * 60 : param.accessTokenLiveTime;
                        js2native.exec("UserInfoPlugin", "setRefreshToken", [param.userID, param.refreshToken, accessTokenLiveTime, store], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            /**
             * @method setUserInfo
             * 设置user的基本信息
             * @param {String} userId 用户id
             * @param {String} loginName 昵称。
             */
            setUserInfo: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.userID)) {
                            param.userID = '';
                        }
                        if (!isDefined(param.pic)) {
                            param.pic = '';
                        }
                        if (!isDefined(param.loginName)) {
                            param.loginName = '';
                        }
                        if (!isDefined(param.forgetGesture)) {
                            param.forgetGesture = '';
                        }
                        if (!isDefined(param.loginOthers)) {
                            param.loginOthers = '';
                        }
                        js2native.exec("UserInfoPlugin", "setUserInfo", [JSON.stringify(param)], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            /**
             * @method getUserInfo
             * 获取user的基本信息
             */
            getUserInfo: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("UserInfoPlugin", "getUserInfo", [], this);
                }
            },
            /**
             * @method setUserLoginInfo
             * 设置用户登录信息
             * @param {String} userId 用户id
             * @param {String} loginName 昵称
             * @param {String} pic 头像图片路径
             * @param {String} accessToken
             */
            setUserLoginInfo: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.gotoMainFlow)) {
                        param.gotoMainFlow = "true";
                    }
                    try {
                        if (!isDefined(param.userID)) throw "userID不能为空";
                        js2native.exec("UserInfoPlugin", "setUserLoginInfo", [
                            [JSON.stringify(param)]
                        ], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            /**
             * @method getAccessToken
             * 获取用户accessToken，并根据参数判断是否弹出登录页。
             * @param {String} [loginAttempt = "true"] 在没有accessToken的时候是否由"客户端"唤起登录页
             */
            getAccessToken: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    var loginAttempt = !isDefined(param.loginAttempt) ? "true" : param.loginAttempt;
                    js2native.exec("UserInfoPlugin", "getAccessToken", [loginAttempt], this);
                }
            },
            /**
             * @method loginResult
             * 已废弃
             * @deprecated
             */
            loginResult: function(param) {
                if (typeof(param) == "object") {
                    var code = !isDefined(param.code) ? "" : param.code;
                    var msg = !isDefined(param.msg) ? "" : param.msg;
                    js2native.exec("UserInfoPlugin", "loginResult", [code, msg], this);
                }
            },
            /**
             * @method getLocalAuthMode
             * 获取本地认证方式 返回长度为3的01字符串标识。
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             * @return 字符串标识从左到右分别表示 刷脸，指纹，手势，如“101”表示用户开启刷脸和手势两种认证方式。
             */
            getLocalAuthMode: function(param) {
                _bindCallback(this, param.success, param.error);
                js2native.exec("UserInfoPlugin", "getLocalAuthMode", [], this);
            },
            /**
             * @method isUsingLocalAuth
             * 是否有本地认证手段
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             * @return 0-没有本地认证 1-有手势认证 2-有指纹认证
             */
            //Todo: 确认：返回值是在success返回的么？
            isUsingLocalAuth: function(param) {
                _bindCallback(this, param.success, param.error);
                js2native.exec("UserInfoPlugin", "isUsingLocalAuth", [], this);
            },
            /**
             * @method logoutUser
             * 登出当前用户
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            logoutUser: function(param) {
                _bindCallback(this, param.success, param.error);
                js2native.exec("UserInfoPlugin", "logoutUser", [], this);
            },
            /**
             * @method loginPwdVerify
             * 登录密码验证
             * @param {String} result 是否验证成功
             */
            loginPwdVerify: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.result)) {
                        param.result = "false";
                    }
                    try {
                        js2native.exec("UserInfoPlugin", "loginPwdVerify", [
                            [JSON.stringify(param)]
                        ], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            }
        },
        //**********************************  UrlProcessPlugin  ************************************//
        /**
         * @class plugins.UrlProcessPlugin
         * Url处理插件
         */
        UrlProcessPlugin: {
            /**
             * @method nextStep
             * 加载下一页
             * @param {Object} param url 加载链接
             * @param {String} [param.pageFlag = "2"] 页面加载方式（1 当前页面加载，原链接入栈，可返回来源；2 新开页面加载；3 当前页面加载并清空栈，不可返回来源；4 新开页面并隐藏标题栏）
             * @param {String} param.titleStr  主标题文字
             * @param {String} param.foreColor 标题栏前景色 （主标题文字颜色/返回键颜色）
             * @param {String} param.subTitle  次标题文字
             * @param {String} param.subTitleColor 次标题文字颜色
             * @param {String} param.backColor 标题栏背景色
             * @param {String} param.tipsNaviType 新页面右上角按钮类型
             * @param {String} param.tipsNaviEventId 右上角按钮点击事件ID
             * @param {String} param.tipsStr 右上角按钮文字
             * @param {String} param.tipsUrl 右上角按钮点击url
             * @param {String} param.tipsPicName 右上角按钮图片地址
             * @param {String} param.prevStepUrlStr 上一步加载url
             * @param {String} param.isLastPage 是否最终页（最终页返回键不显示）
             * @param {String} param.needLogin 是否需要登录
             * @param {String} param.closeCount 关闭按钮显示层级（从多少层开始显示关闭按钮）
			 * @param {String} param.ignoreWL   0:保持目前的逻辑，都要过白名单检查 1：不过白名单检查
			 * @param {String} param.statusColor  "":按目前逻辑不变，“0”：白色  “1”：黑色
             */
            nextStep: function(param) {
                var _titleStr = !isDefined(param.titleStr) ? "详情" : param.titleStr;
                var tipsStr = !isDefined(param.tipsStr) ? "" : param.tipsStr;
                var tipsUrl = !isDefined(param.tipsUrl) ? "" : param.tipsUrl;
                if (param.needLogin == false || !isDefined(param.needLogin)) {
                    param.needLogin = "00";
                } else if (param.needLogin == true) {
                    param.needLogin = "11";
                }
                var nextStepParam = {
                    url: !isDefined(param.url) ? "" : param.url,
                    pageFlag: !isDefined(param.pageFlag) ? "2" : param.pageFlag,
                    title: {
                        mainTitle: _titleStr,
                        mainTitleColor: !isDefined(param.foreColor) ? "#ffd5b671" : param.foreColor,
                        subTitle: !isDefined(param.subTitle) ? "" : param.subTitle,
                        subTitleColor: !isDefined(param.subTitleColor) ? "#ffd5b671" : param.subTitleColor,
                        backgroundColor: !isDefined(param.backColor) ? "#ff453B39" : param.backColor,
                        pageCtrlBtn: param.pageCtrlBtn
                    },
                    tipNavi: {
                        type: param.tipsNaviType,
                        eventId: param.tipsNaviEventId,
                        naviName: tipsStr,
                        naviUrl: tipsUrl,
                        picName: !isDefined(param.tipsPicName) ? "" : param.tipsPicName
                    },
                    prevStepUrlStr: !isDefined(param.prevStepUrlStr) ? "" : param.prevStepUrlStr,
                    isLastPage: !isDefined(param.isLastPage) ? "false" : param.isLastPage,
                    needLogin: param.needLogin,
                    closeCount: !isDefined(param.closeCount) ? "2" : param.closeCount,
					ignoreWL: !isDefined(param.ignoreWL) ? "0" : param.ignoreWL,
					statusColor: !isDefined(param.statusColor) ? "" : param.statusColor
                };
                js2native.exec("UrlProcessPlugin", "nextStep", [JSON.stringify(nextStepParam)], this);
            },
            /**
             * @method preview
             * 产品说明书 下载PDF 未实测
             * @param url PDF文件地址
             *        reloadUrl 重新加载地址
             *        title 页面标题
             */
            //Todo: 方法名比较怪，需讨论
            preview: function(url, reloadUrl, title) {
                js2native.exec("UrlProcessPlugin", "preview", [url, reloadUrl, title], this);

            },
            /**
             * @method prevStep
             * 跳转到上一页
             * @param url 上个页面加载url
             */
            prevStep: function(url) {
                js2native.exec("UrlProcessPlugin", "prevStep", [url], this);

            },
            /**
             * @method popToRoot
             * 跳转到根页面
             * @param url 到达根页面需要加载的url
             */
            //Todo: 确认：url是否是跳转到root之后所显示的页面？
            popToRoot: function(url) {
                js2native.exec("UrlProcessPlugin", "popToRoot", [url], this);

            },
            /**
             * @method popToTab
             * 跳转到根页面并且切tab页
             * @param {int} param.tabId  指定切换到的tab页id，0表示第一页
             */
            popToTab: function(param) {
                var _param = null;
                if (typeof(param) != "object") {
                    _param = {
                        tabId: '0'
                    };
                } else {
                    _param = param;
                }
                js2native.exec("UrlProcessPlugin", "popToTab", [JSON.stringify(_param)], this);

            },
            /**
             * @method popToLevel
             * 到达指定层级页面
             * @param param 兼容旧接口，如果传入的是数字，则当页码用。
             * @param {String} param.reloadURL  指定要加载的url
             */
            popToLevel: function(param) {
                var _param = null;
                if (typeof(param) != "object") {
                    _param = {
                        count: param,
                        reloadURL: ""
                    };
                } else {
                    _param = param;
                }
                js2native.exec("UrlProcessPlugin", "popToPage", [JSON.stringify(_param)], this);

            },
            /**
             * @method popPage
             * 自上而下抛掉指定层数的页面
             * @param param 兼容旧接口，如果传入的是数字，则当页码用。
             * @param {String} param.reloadURL  指定要加载的url
             */
            popPage: function(param) {
                var _param = null;
                if (typeof(param) != "object") {
                    _param = {
                        count: param,
                        reloadURL: ""
                    };
                } else {
                    _param = param;
                }
                js2native.exec("UrlProcessPlugin", "popPage", [JSON.stringify(_param)], this);

            },
            /**
             * @method reload
             * 重新加载WebView目标页面
             * @param 无
             */
            reload: function() {
                js2native.exec("UrlProcessPlugin", "reload", [], this);

            },
            /**
             * @method setBounce
             * 设置WebView是否支持上拉、下拉
             * @param bounceType 【0-不支持bounce，1-支持上方的，2-支持下方的，3-上下都支持】 //已过期 2017-07-20 zhengmj
             * @param param.bounceTop 【style="default"-默认 "airplane"-小飞机】
             * @param param.bounceBottom 【style="default"-默认】
             */
            setBounce: function(param) {
                if (typeof(param) == "object") {
                    if (isDefined(param.type)) {
                        if (!isDefined(param.bounceTop) && !isDefined(param.bounceBottom)) {
                            if (parseInt(param.type) == 1 || parseInt(param.type) == 3) {
                                param.bounceTop = { style: "default" };
                            }
                            if (parseInt(param.type) == 2 || parseInt(param.type) == 3) {
                                param.bounceBottom = { style: "default" };
                            }
                        }
                    } else {
                        //nothing
                    }
                    var _param = param;
                    _param.success = null;
                    _param.error = null;
                    js2native.exec("UrlProcessPlugin", "setBounce", [JSON.stringify(_param)], this);
                }

            },
            /**
             * @method reset
             * WebView上拉下拉的刷新动画复位
             * @param 无
             */
            reset: function(param) {
				if (typeof(param) == "object") {
					   js2native.exec("UrlProcessPlugin", "reset", [JSON.stringify(param)], this);
				}else{
					   js2native.exec("UrlProcessPlugin", "reset", [], this);
				}
            },
            /**
             * @method addNaviList
             * 添加右上角Navi按钮 单个按钮则显示在右上角 多个按钮收进列表（类似微信）
             * @param {String} param naviList 右上角按钮列表 [{"type":"TR01","eventId":"","naviName":"","naviUrl":"","picName":""},……]
             *                             （ type-->按钮类型 eventId--> 按钮点击事件ID naviName--> 按钮文字 naviUrl--> 按钮点击加载url picName-->按钮图片地址）
             */
            addNaviList: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    var naviList = !isDefined(param.naviList) ? [] : param.naviList;
                    js2native.exec("UrlProcessPlugin", "addNaviList", [JSON.stringify(naviList)], this);
                }

            },
            /**
             * @method clearNaviList
             * 清除右上角Navi按钮
             * @param 无
             */
            clearNaviList: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("UrlProcessPlugin", "clearNaviList", [], this);
                }

            },
            /**
             * @method setTitle
             * 设置页面标题
             * @param title 页面标题
             */
            setTitle: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("UrlProcessPlugin", "setTitle", [param.title], this);
                }

            }
        },
        /**
         * @class plugins.SharePlugin
         */

        SharePlugin: {
            share: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    var shareInfo = !isDefined(param.shareInfo) ? [] : param.shareInfo;
                    js2native.exec("SharePlugin", "share", [JSON.stringify(shareInfo)], this);
                }
            },
			shareScreen: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    var shareInfo = !isDefined(param.shareInfo) ? [] : param.shareInfo;
                    js2native.exec("SharePlugin", "shareScreen", [JSON.stringify(shareInfo)], this);
                }
            }

        },
        /**
         * @class plugins.DatePickerPlugin
         * 日期选择插件（日期控件）
         */
        DatePickerPlugin: {
            /**
             * @method show
             * 显示日期控件
             * @param {Object} param
             * @param {String} [param.date = '2016-01-01'] 当前默认选中的日期
             * @param {String} [param.style = 'default'] 日期控件样式  default:默认样式 xcd:兴财道样式 bottom:底部布局样式
             * @param {String} [param.rangeBegin = ''] 起始日期
             * @param {String} [param.rangeEnd = ''] 结束日期
             * @param {String} [param.cols = '3'] 显示列数 年月日或者年月
             */
            show: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    var date = !isDefined(param.date) ? "2016-01-01" : param.date;
                    var style = !isDefined(param.style) ? "default" : param.style;
                    var rangeBegin = !isDefined(param.rangeBegin) ? "" : param.rangeBegin;
                    var rangeEnd = !isDefined(param.rangeEnd) ? "" : param.rangeEnd;
                    var cols = !isDefined(param.cols) ? "3" : param.cols;
                    js2native.exec("DatePickerPlugin", "show", [style, cols, date, rangeBegin, rangeEnd], this);
                }

            }
        },
        //**********************************  TimePickerPlugin  ************************************//
        TimePickerPlugin: {
            /**
             * @method show
             * 显示时间控件
             * @param hour 小时制，12和24，代表12小时制或24小时制。如果是12小时制需额外加入一个滚动选择框AM或PM
             *        time 初始时间
             *        style 样式,默认default
             *        rangeBegin 起始时间
             *        rangeEnd 结束时间
             *        cols 2显示时分，3显示时分秒
             *        amPm 仅当hour为12时有效，默认AM，用于初始化指定12小时制的AM或PM
             *        step 步进值，一般情况下是1,1,1，代表三个栏位都按1增长，
             *             如果是1,5的情况就是第一个栏位步进是1，第二个栏位步进是5
             */
            show: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    var hour = !isDefined(param.hour) || param.hour != "12" || param.hour != "24" ? "24" : param.hour;
                    var time = !isDefined(param.time) ? "00:00:00" : param.time;
                    var style = !isDefined(param.style) ? "default" : param.style;
                    var rangeBegin = !isDefined(param.rangeBegin) ? "" : param.rangeBegin;
                    var rangeEnd = !isDefined(param.rangeEnd) ? "" : param.rangeEnd;
                    var cols = !isDefined(param.cols) ? "3" : param.cols;
                    var amPm = !isDefined(param.amPm) ? "AM" : param.amPm;
                    var step = !isDefined(param.step) ? "1,1,1" : param.step;
                    js2native.exec("TimePickerPlugin", "show", [style, cols, time, hour, amPm, step, rangeBegin, rangeEnd], this);
                }

            }
        },
        //**********************************  LocalStoragePlugin  ********************************//
        LocalStoragePlugin: {
            /**
             * @method put
             * 设置一组数据（键值对方式）到文件中
             * @param {String[String]} data {
             *      key: {String} [] 键
             *      value: {String} [] 值
             * }
             */
            put: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (param.data.length == 0) {
                            throw "data 不能为空";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    js2native.exec("LocalStoragePlugin", "put", [JSON.stringify(param.data)], this);
                }

            },
            /**
             * @method get
             * 通过键取出文件中储存的值
             * @param {Array} data [] key值的数组
             *
             */
            get: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.keys) || param.keys.length == 0) {
                            throw "keys 不能为空";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    js2native.exec("LocalStoragePlugin", "get", [JSON.stringify(param.keys)], this);
                }

            },
            /**
             * @method del
             * 通过键删除文件中储存的值
             * @param {Array} data [] key值的数组
             *
             */
            del: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.keys) || param.keys.length == 0) {
                            throw "keys 不能为空";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    js2native.exec("LocalStoragePlugin", "del", [JSON.stringify(param.keys)], this);
                }
            }

        },
        /**
         * @class plugins.MemStoragePlugin
         */
        MemStoragePlugin: {
            /**
             * @method put
             * 在内存中存放数据。
             * @param {Object} data 需要存放的数据的字典（key-value）
             */
            put: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (param.data.length == 0) {
                            throw "data 不能为空";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    js2native.exec("MemStoragePlugin", "put", [JSON.stringify(param.data)], this);
                }

            },
            /**
             * @method put
             * 取出内存中对应key的数据
             * @param {Array} data [] key的数组
             */
            get: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.keys) || param.keys.length == 0) {
                            throw "keys 不能为空";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    js2native.exec("MemStoragePlugin", "get", [JSON.stringify(param.keys)], this);
                }

            },
            /**
             * @method del
             * 删除内存中对应key的数据
             * @param {Array} data [] key的数组
             */
            del: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.keys) || param.keys.length == 0) {
                            throw "keys 不能为空";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    js2native.exec("MemStoragePlugin", "del", [JSON.stringify(param.keys)], this);
                }
            },
            /**
             * @method clear
             * 删除内存中所有数据
             * @param {Object} param
             * @param {Function} param.success 成功回调函数
             * @param {Function} param.error 失败回调函数
             */
            clear: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                }
                js2native.exec("MemStoragePlugin", "clear", [], this);
            }
        },
        /**
         * @class plugins.TalkingDataPlugin
         * 数据埋点插件
         */
        TalkingDataPlugin: {
            trackEvent: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.eventId)) {
                            throw "event id invalid";
                        }
                        var eventLabel = !isDefined(param.eventLabel) ? "" : param.eventLabel;
                        var eventData = !isDefined(param.eventData) ? "{}" : param.eventData;
                        js2native.exec("TalkingDataPlugin", "trackEvent", [param.eventId, eventLabel, eventData], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            trackPage: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.pageName)) {
                            throw "pageName invalid";
                        }
                        js2native.exec("TalkingDataPlugin", "trackPage", [param.pageName], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            trackPageBegin: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.pageName)) {
                            throw "pageName invalid";
                        }
                        js2native.exec("TalkingDataPlugin", "trackPageBegin", [param.pageName], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            trackPageEnd: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.pageName)) {
                            throw "pageName invalid";
                        }
                        js2native.exec("TalkingDataPlugin", "trackPageEnd", [param.pageName], this);
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                    }
                }
            },
            getDeviceId: function() {
                js2native.exec("TalkingDataPlugin", "getDeviceId", [], this);
            },
            setExceptionReportEnabled: function(enabled) {
                if (!isDefined(enabled)) {
                    enabled = true;
                }
                js2native.exec("TalkingDataPlugin", "setExceptionReportEnabled", [enabled], this);
            }
        },
        /**
         * @class plugins.CachePlugin
         */
        CachePlugin: {
            /**
             * @method clear
             * 清除缓存
             * @param 无
             */
            clear: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("CachePlugin", "clear", [], this);
                }
            },
            /**
             * @method cacheSize
             * 计算缓存大小
             * @param 无
             */
            cacheSize: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("CachePlugin", "cacheSize", [], this);
                }
            }
        },
        /**
         * @class plugins.TagPlugin
         */
        TagPlugin: {
            addTag: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.tagNames) || param.tagNames.length == 0) {
                        if (isDefined(param.error)) {
                            param.error("tagNames can't not be null");
                        }
                        return;
                    }
                    js2native.exec("TagPlugin", "addTag", [JSON.stringify(param.tagNames)], this);
                }
            },
            delTag: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.tagNames) || param.tagNames.length == 0) {
                        if (isDefined(param.error)) {
                            param.error("tagNames can't not be null");
                        }
                        return;
                    }
                    js2native.exec("TagPlugin", "delTag", [JSON.stringify(param.tagNames)], this);
                }
            },
            listTag: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("TagPlugin", "listTag", [], this);
                }
            }
        },
        /**
         * @class plugins.UpdatePlugin
         */
        UpdatePlugin: {
            /**
             * @method check
             * 检查版本更新
             * @param 无
             */
            check: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("UpdatePlugin", "check", [param.tagName], this);
                }
            }
        },
        /**
         * @class plugins.UIPlugin
         */
        UIPlugin: {
            rebuild: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.naviRoot)) {
                        if (isDefined(param.error)) {
                            param.error("naviRoot can not be empty");
                        }
                        return;
                    }
                    js2native.exec("UIPlugin", "rebuild", [param.naviRoot], this);
                }
            },
            addNavi: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    try {
                        if (!isDefined(param.naviRoot)) {
                            throw "naviRoot can not be null";
                        }
                        if (!isDefined(param.naviTree) || param.naviTree.length == 0) {
                            throw "naviTree can not be null";
                        }
                    } catch (_ex) {
                        if (isDefined(param.error)) {
                            param.error(_ex);
                        }
                        return;
                    }
                    var order = !isDefined(param.order) ? "1.1" : param.order;

                    js2native.exec("UIPlugin", "addNavi", [param.naviRoot, order, param.naviTree], this);
                }
            },
            reset: function(param) {
                if (typeof(param) == "object") {
                    _bindCallback(this, param.success, param.error);
                    if (!isDefined(param.naviRoot)) {
                        if (isDefined(param.error)) {
                            param.error("naviRoot can not be empty");
                        }
                        return;
                    }
                    js2native.exec("UIPlugin", "reset", [param.naviRoot], this);
                }
            }
        },
        /**
         * @class plugins.AvataPlugin
         */
        AvataPlugin: {
            /**
             * @method getAvata
             * 获取头像、头像裁剪、头像上传
             * @param captureFrame 头像裁剪框 0 圆框 1 方框
             *        storeUrl 上传地址
             */
            getAvata: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.storeUrl)) {
                    if (isDefined(param.error)) {
                        param.error("store url can not be null");
                    }
                    return;
                }
                param.captureFrame = !isDefined(param.captureFrame) ? "0" : param.captureFrame;
                param.uploadFieldName = !isDefined(param.uploadFieldName) ? "file" : param.uploadFieldName;
                param.requestData = !isDefined(param.requestData) ? {} : param.requestData;
                js2native.exec("AvataPlugin", "getAvata", [JSON.stringify(param)], this);
            }
        },
        /**
         * @class plugins.PicUploadPlugin
         */
        PicUploadPlugin: {
            /**
             * @method show
             * 图片上传
             * @param storeUrl 图片上传地址
             *        timeout 超时时间
             *		  quality 图片压缩质量，low:30,middle:50,high:80
             */
            show: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.storeUrl)) {
                    if (isDefined(param.error)) {
                        param.error("store url can not be null");
                    }
                    return;
                }
                if (!isDefined(param.timeout)) {
                    param.timeout = 30;
                }
                if (!isDefined(param.quality)) {
                    param.quality = "middle";
                }
                js2native.exec("PicUploadPlugin", "show", [JSON.stringify(param)], this);
            }
        },
        /**
         * @class plugins.QRCodeScanPlugin
         */
        QRCodeScanPlugin: {
            scan: function(param) {
                _bindCallback(this, param.success, param.error);
                var mode = !isDefined(param.mode) ? "default" : param.mode;
                js2native.exec("QRCodeScanPlugin", "scan", [mode], this);
            }
        },
        /**
         * @class plugins.SelectPlugin
         */
        SelectPlugin: {
            /**
             * @method select
             * 列表选择
             * @param  style 列表选择样式
             *         options 选项列表数组  [{name:'苍井老师',value:'oyi sola'},……] name-->界面展示元素 value--> 选择项回调值
             */
            select: function(param) {
                _bindCallback(this, param.success, param.error);
                var style = !isDefined(param.style) ? "default" : param.style;
                var options = !isDefined(param.options) ? [] : param.options;
                js2native.exec("SelectPlugin", "select", [style, JSON.stringify(options)], this);
            }
        },
        /**
         * @class plugins.GeographyPlugin
         */
        GeographyPlugin: {
            /**
             * @method selectCity
             * 选择城市
             * @param {Object} param hotCities 热门城市数组['福州',……]
             */
            selectCity: function(param) {
                _bindCallback(this, param.success, param.error);
                var hotCities = !isDefined(param.hotCities) ? [] : param.hotCities;
                var style = !isDefined(param.style) ? "default" : param.style;
                js2native.exec("GeographyPlugin", "selectCity", [style, JSON.stringify(hotCities)], this);
            },
            /**
             * @method selectArea
             * 选择区域
             * @param {Object} param
             * @param {String} param.provinceUrl 省份数据获取url
             * @param {String} param.childAreaUrl 市区县数据获取url
             * @param {Number} param.depth 区域数据的层级
             */
            selectArea: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.provinceUrl) || param.provinceUrl.length == 0) {
                        throw "provinceUrl can not be null";
                    }
                    if (!isDefined(param.childAreaUrl) || param.childAreaUrl.length == 0) {
                        throw "childAreaUrl can not be null";
                    }
                } catch (_ex) {
                    if (isDefined(param.error)) {
                        param.error(_ex);
                    }
                }
                if (!isDefined(param.depth)) {
                    param.depth = 3;
                }
                var style = !isDefined(param.style) ? "default" : param.style;
                js2native.exec("GeographyPlugin", "selectArea", [JSON.stringify(param)], this);
            },
            /**
             * @method getSelectedCity
             * 获取当前选择的城市名
             * @param {Object} param
             */
            getSelectedCity: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("GeographyPlugin", "getSelectedCity", [], this);
                }
            }
        },
        /**
         * @class plugins.PinPwdVerifyPlugin
         */
        //Todo: Pin校验插件？？？怎么用的？
        PinPwdVerifyPlugin: {
            verify: function(param) {
                _bindCallback(this, param.success, param.error);
                var pwdPadModel = !isDefined(param.pwdPadModel) ? {
                    style: "default",
                    visualResponse: !isDefined(param.pwdPadModel.visualResponse) ? "true" : param.pwdPadModel.visualResponse,
                    labelText: "兴业数金安全键盘",
                    titleText: "输入支付密码",
                    forgetPwdUrl: !isDefined(param.pwdPadModel.forgetPwdUrl) ? "" : param.pwdPadModel.forgetPwdUrl
                } : param.pwdPadModel;
                if (!isDefined(param.pwdPadModel.animStyle)) {
                    param.pwdPadModel.animStyle = "BottomUp";
                }
                if (!isDefined(param.pwdPadModel.closeStyle)) {
                    param.pwdPadModel.closeStyle = "close";
                }
                try {
                    if (!isDefined(param.hostInfo)) {
                        throw "hostInfo can not be null";
                    }
                    if (!isDefined(param.hostInfo.url)) {
                        throw "url can not be null";
                    }
                    if (!isDefined(param.hostInfo.pwdFieldName)) {
                        throw "password field name can not be null";
                    }
                    if (!isDefined(param.hostInfo.timeout)) {
                        param.hostInfo.timeout = 30;
                    }
                    if (!isDefined(param.hostInfo.timeoutMsg)) {
                        param.hostInfo.timeoutMsg = "对不起,服务器有点忙,请稍后再试";
                    }
                    if (!isDefined(param.hostInfo.method)) {
                        param.hostInfo.method = "GET";
                    }
                    if (!isDefined(param.hostInfo.parameters)) {
                        param.hostInfo.parameters = [];
                    }
                } catch (_ex) {
                    if (isDefined(param.error)) {
                        param.error(_ex);
                    }
                }
                var random = !isDefined(param.random) ? "true" : param.random;
                var algorithm = !isDefined(param.algorithm) ? {
                    algName: "SHA1"
                } : param.algorithm;
                js2native.exec("PinPwdVerifyPlugin", "verify", [JSON.stringify(pwdPadModel), JSON.stringify(param.hostInfo), JSON.stringify(algorithm), random], this);
            }
        },
		/**
		 *@class plugins.HKEPlugin
		 *云证通插件
		 */
		 HKEPlugin:{
			 queryCertInfo:function(param){
				_bindCallback(this, param.success, param.error);
				try{
					if (!isDefined(param.name)) {
						throw "name can not be empty";
					}
					if (!isDefined(param.identityTypeCode)) {
						throw "identityTypeCode can not be empty";
					}
					if (!isDefined(param.identityNumber)) {
						throw "identityNumber can not be empty";
					}
					if (!isDefined(param.phoneNumber)) {
						throw "phoneNumber can not be empty";
					}
					if (!isDefined(param.randomSignUrl)) {
						throw "randomSignUrl can not be empty";
					}
				}catch(_ex){
					if (isDefined(param.error)) {
						param.error(_ex);
					} 
					return;
			   }
				js2native.exec("HKEPlugin", "queryCertInfo", [JSON.stringify(param)], this);
			 },
			 downloadCert:function(param){
				 _bindCallback(this, param.success, param.error);
				try{
					if (!isDefined(param.name)) {
						throw "name can not be empty";
					}
					if (!isDefined(param.identityTypeCode)) {
						throw "identityTypeCode can not be empty";
					}
					if (!isDefined(param.identityNumber)) {
						throw "identityNumber can not be empty";
					}
					if (!isDefined(param.phoneNumber)) {
						throw "phoneNumber can not be empty";
					}
					if (!isDefined(param.randomSignUrl)) {
						throw "randomSignUrl can not be empty";
					}
			   }catch(_ex){
					if (isDefined(param.error)) {
						param.error(_ex);
					} 
					return;
			   }
				js2native.exec("HKEPlugin", "downloadCert", [JSON.stringify(param)], this);
			 },
			 setPin:function(param){
				_bindCallback(this, param.success, param.error);
                try {
					if (!isDefined(param.name)) {
						throw "name can not be empty";
					}
					if (!isDefined(param.identityTypeCode)) {
						throw "identityTypeCode can not be empty";
					}
					if (!isDefined(param.identityNumber)) {
						throw "identityNumber can not be empty";
					}
					if (!isDefined(param.phoneNumber)) {
						throw "phoneNumber can not be empty";
					}
					if (!isDefined(param.randomSignUrl)) {
						throw "randomSignUrl can not be empty";
					}
                    if (!isDefined(param.pwdKey1)) {
                        throw "pwd key 1 can not be empty";
                    }
                    if (!isDefined(param.pwdKey2)) {
                        throw "pwd key 2 can not be empty";
                    }
                } catch (_ex) {
                    if (isDefined(param.error)) {
                        param.error(_ex);
                    }
                    return;
                }
               
                js2native.exec("HKEPlugin", "setPin", [JSON.stringify(param)], this);
				
			 },
			 modifyPin:function(param){
				   _bindCallback(this, param.success, param.error);
				  try {
                    if (!isDefined(param.pwdKey)) {
                        throw "pwd key 1 can not be empty";
                    }
                    if (!isDefined(param.pwdKey1)) {
                        throw "pwd key 2 can not be empty";
                    }
					 if (!isDefined(param.pwdKey2)) {
                        throw "pwd key 3 can not be empty";
                    }
					if (!isDefined(param.name)) {
						throw "name can not be empty";
					}
					if (!isDefined(param.identityTypeCode)) {
						throw "identityTypeCode can not be empty";
					}
					if (!isDefined(param.identityNumber)) {
						throw "identityNumber can not be empty";
					}
					if (!isDefined(param.phoneNumber)) {
						throw "phoneNumber can not be empty";
					}
					if (!isDefined(param.randomSignUrl)) {
						throw "randomSignUrl can not be empty";
					}
                } catch (_ex) {
                    if (isDefined(param.error)) {
                        param.error(_ex);
                    }
                    return;
                }
                js2native.exec("HKEPlugin", "modifyPin", [JSON.stringify(param)], this);
			 },
		     getDeviceID:function(param) {
				 _bindCallback(this, param.success, param.error);
				js2native.exec("HKEPlugin", "getDeviceID", [], this);
			 }
		 },
		 
        /**
         * @class plugins.PDFReaderPlugin
         * PDF阅读器插件
         */
        PDFReaderPlugin: {
            /**
             * @method load
             * 下载PDF文件，并启动查看PDF文件
             * @param {Object} param
             * @param {String} param.url PDF文件下载地址
             */
            load: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.url)) {
                    if (isDefined(param.error)) {
                        param.error("url can not be empty");
                    }
                    return;
                }
                js2native.exec("PDFReaderPlugin", "load", [param.url], this);
            }
        },
        /**
         * @class plugins.TextToSpeechPlugin
         */
        TextToSpeechPlugin: {
            speek: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.content)) {
                    if (isDefined(param.error)) {
                        param.error("content can not be empty");
                    }
                    return;
                }
                var voiceType = !isDefined(param.voiceType) ? "default" : param.voiceType;
                js2native.exec("TextToSpeechPlugin", "speek", [param.content, voiceType], this);
            }
        },
        /**
         * @class plugins.SpeechToTextPlugin
         */
        SpeechToTextPlugin: {
            listen: function(param) {
                _bindCallback(this, param.success, param.error);
                js2native.exec("SpeechToTextPlugin", "listen", [], this);
            }
        },
        /**
         * @class plugins.HFSecurityRegisterPlugin
         */
        HFSecurityRegisterPlugin: {
            /**
             * @method load
             * 启动开户插件
             * @param {Object} param source_id/thrid_channel/start_type/start_ramark 华福提供数据 用于渠道校验
             *                      mobile_no 手机号
             *                      idno 身份证号
             *                      bankCard 银行卡号
             *                      url 华福启动链接（华福提供 区分测试环境/开发环境）
             */
            load: function(param) {
                _bindCallback(this, param.success, param.error);
                js2native.exec("HFSecurityRegisterPlugin", "load", [JSON.stringify(param)], this);
            }
        },
        /**
         * @class plugins.ARPlugin
         */
        ARPlugin: {
            load: function(param) {
                _bindCallback(this, param.success, param.error);
                js2native.exec("ARPlugin", "load", [JSON.stringify(param)], this);
            }
        },
        /**
         * @class plugins.MapPlugin
         */
        MapPlugin: {
            /**
             * @method load
             * 启动地图导航
             * @param {Object} param name 网点名称
             */
            load: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.name)) {
                    param.name = "";
                }
                js2native.exec("MapPlugin", "load", [JSON.stringify(param)], this);
            },
            locate: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.name)) {
                    param.name = "";
                }
                try {
                    if (!isDefined(param.longitude)) {
                        throw "经度值不合法";
                    } else {
                        try {
                            parseFloat(param.longitude);
                        } catch (ex) {
                            throw "经度值不合法";
                        }
                    }
                    if (!isDefined(param.latitude)) {
                        throw "纬度值不合法";
                    } else {
                        try {
                            parseFloat(param.latitude);
                        } catch (ex) {
                            throw "纬度值不合法";
                        }
                    }
                    js2native.exec("MapPlugin", "locate", [JSON.stringify(param)], this);
                } catch (e) {
                    if (isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.GalleryPlugin
         */
        GalleryPlugin: {
            /**
             * @method load
             * 图片展示
             * @param {Object} param picList 图片url数组 []
             */
            load: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.picList)) {
                    if (isDefined(param.error)) {
                        param.error("picList can not be empty");
                    }
                    return;
                }
                if (!isDefined(param.index)) {
                    param.index = 0;
                }
                js2native.exec("GalleryPlugin", "load", [JSON.stringify(param)], this);
            }
        },
        /**
         * @class plugins.OCRPlugin
         */
        OCRPlugin: {
            /**
             * @method scanIdCard
             * 扫描身份证
             * @param {Object} param   showResult 是否显示结果页
             *                       isOnline  是否在线识别
             *                       scanMode  扫描模式（1 正面 2 反面）
             *                       postUrl  在线模式上传地址
             *                       isAutoScan  是否自动扫描
             */
            scanIdCard: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.showResult)) {
                    param.showResult = true;
                }
                if (!isDefined(param.isOnline)) {
                    param.isOnline = true;
                }
                if (!isDefined(param.scanMode)) {
                    param.scanMode = 1;
                }
                if (param.isOnline && !isDefined(param.postUrl)) {
                    if (isDefined(param.error)) {
                        param.error("postUrl can not be empty");
                        return;
                    }
                }
                if (!isDefined(param.isAutoScan)) {
                    param.isAutoScan = false;
                }
                js2native.exec("OCRPlugin", "scanIdCard", [JSON.stringify(param)], this);
            },
            /**
             * @method scanBankCard
             * 扫描银行卡
             * @param {Object} param     showResult 是否显示结果页
             *                         isOffline  是否离线
             *                          scanMode  扫描模式（1 单面 2 双面）
             *                          postUrl  非离线模式上传地址
             *                       isAutoScan  是否自动扫描
             */
            scanBankCard: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.showResult)) {
                    param.showResult = true;
                }
                if (!isDefined(param.isOffline)) {
                    param.isOffline = true;
                }
                if (!isDefined(param.scanMode)) {
                    param.scanMode = 1;
                }
                if (!param.isOffline && !isDefined(param.postUrl)) {
                    if (isDefined(param.error)) {
                        param.error("postUrl can not be empty");
                        return;
                    }
                }
                if (!isDefined(param.isAutoScan)) {
                    param.isAutoScan = false;
                }
                js2native.exec("OCRPlugin", "scanBankCard", [JSON.stringify(param)], this);
            },
        },
        /**
         * @class plugins.CollectionPlugin
         */
        CollectionPlugin: {
            /**
             * @method list
             * 显示收藏列表
             * @param {Object} param title为收藏列表页title数据、tipNavi为列表页右上角按钮数据 （详见UrlProcessPlugin NextStep）
             */
            list: function(param) {
                _bindCallback(this, param.success, param.error);
                var collectionParam = {
                    title: {
                        mainTitle: !isDefined(param.titleStr) ? "我的收藏" : param.titleStr,
                        mainTitleColor: !isDefined(param.foreColor) ? "#ffd5b671" : param.foreColor,
                        subTitle: !isDefined(param.subTitle) ? "" : param.subTitle,
                        subTitleColor: !isDefined(param.subTitleColor) ? "#ffd5b671" : param.subTitleColor,
                        backgroundColor: !isDefined(param.backColor) ? "#ff453B39" : param.backColor
                    },
                    tipNavi: {
                        type: param.tipsNaviType,
                        eventId: param.tipsNaviEventId,
                        naviName: !isDefined(param.tipsStr) ? "" : param.tipsStr,
                        naviUrl: param.tipsUrl,
                        picName: !isDefined(param.tipsPicName) ? "" : param.tipsPicName
                    }
                }
                js2native.exec("CollectionPlugin", "list", [JSON.stringify(collectionParam)], this);
            },
            /**
             * @method add
             * 添加收藏
             * @param id 收藏ID
             *        content 收藏内容
             */
            add: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.id)) {
                    if (isDefined(param.error)) {
                        param.error("id can not be empty");
                    }
                    return;
                }
                js2native.exec("CollectionPlugin", "add", [param.id, param.content], this);
            },
            /**
             * @method isCollect
             * 是否被收藏
             * @param id 要查询的收藏ID
             */
            isCollect: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.id)) {
                    if (isDefined(param.error)) {
                        param.error("id can not be empty");
                    }
                    return;
                }
                js2native.exec("CollectionPlugin", "isCollect", [param.id], this);
            },
            /**
             * @method del
             * 删除对应的ID的收藏内容
             * @param idList 要删除的收藏ID列表 ["Fund_010001",……]
             */
            del: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.idList) || param.idList.length == 0) {
                    if (isDefined(param.error)) {
                        param.error("idList can not be empty");
                    }
                    return;
                }
                js2native.exec("CollectionPlugin", "del", [JSON.stringify(param.idList)], this);
            }
        },
        /**
         * @class plugins.IMPlugin
         */
        IMPlugin: {
            /**
             * @method IMPlugin.login
             * login前需要先通过手机号查询一次getSingleLoginInfo，获得uid和signKey
             * @param {Object} param
             * @param {String} param.uid 用户id
             * @param {String} param.signKey
             */
            login: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.uid)) {
                        throw "uid can not be empty";
                    }
                    if (!isDefined(param.signKey)) {
                        throw "signKey can not be empty";
                    }
                } catch (e) {
                    if (isDefined(param.error)) {
                        param.error(e);
                    }
                    return;
                }
                js2native.exec("IMPlugin", "login", [JSON.stringify(param)], this);
            },
            /**
             * @method IMPlugin.chat
             * 打开进入聊天窗口
             * @param {Object} param
             * @param {String} param.chatType 0:单聊 1:活动群 2:普通群
             * @param {String} param.chatId 聊天ID（若是单聊，则为对方uid）
             */
            chat: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.chatType)) {
                        throw "chat type can not be empty";
                    }
                    if (!isDefined(param.chatId)) {
                        throw "chatId can not be empty";
                    }
                    js2native.exec("IMPlugin", "chat", [JSON.stringify(param)], this);
                } catch (e) {
                    if (isDefined(param.error)) {
                        param.error(e);
                    }
                }
            },
            /**
             * @method IMPlugin.getOrganizeChatList
             * 打开进入群组列表
             */
            getOrganizeChatList: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    js2native.exec("IMPlugin", "getOrganizeChatList", [], this);
                } catch (e) {
                    if (isDefined(param.error)) {
                        param.error(e);
                    }
                }
            },

            /**
             * @method IMPlugin.getOrganizeChatList
             * 打开进入群组列表
             */
            getChatMessageList: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    js2native.exec("IMPlugin", "getChatMessageList", [], this);
                } catch (e) {
                    if (isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.FaceRecognizePlugin
         */
        FaceRecognizePlugin: {
            /**
             * @method upload
             * 图片上传
             * @param {Object} param userId 用户ID
             *                      url 上传地址
             *                      type 图片类型
             */
            upload: function(param) {
                _bindCallback(this, param.success, param.error);
                if (!isDefined(param.url)) {
                    if (isDefined(param.error)) {
                        param.error("url can not be empty");
                    }
                }
                if (!isDefined(param.type)) {
                    param.type = "0";
                }
                js2native.exec("FaceRecognizePlugin", "upload", [JSON.stringify(param)], this);
            },
            /**
             * @method livenessCheck
             * 活体检测
             * @param {Object} param userId 用户ID
             *                      url 上传二进制文件检测地址
             *                      motion 活体检测动作序列 [{"operate":"mouth","words":"让我看看你有没有蛀牙"}，……] operate --> 操作名称 words--> 界面显示文字
             *                      complexity 复杂度
             *                      soundNotice 默认是否语音提示
             */
            livenessCheck: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.url)) {
                        throw "url can not be empty";
                    }
                    if (!isDefined(param.motion) || param.motion.length == 0) {
                        throw "motions can not be empty";
                    }
                    if (!isDefined(param.complexity)) {
                        param.complexity = "hell";
                    }
                    if (!isDefined(param.soundNotice)) {
                        param.soundNotice = true;
                    }
                    js2native.exec("FaceRecognizePlugin", "livenessCheck", [JSON.stringify(param)], this);
                } catch (e) {
                    if (isDefined(param.error)) {
                        param.error(e);
                    }
                }
            },
            /**
             * @method livenessCheck
             * 设置刷脸登录
             * @param {Object} param  url 上传二进制文件检测地址
             *                      motion 活体检测动作序列 [{"operate":"mouth","words":"让我看看你有没有蛀牙"}，……] operate --> 操作名称 words--> 界面显示文字
             *                      complexity 复杂度
             *                      soundNotice 默认是否语音提示
             *                      style 样式
             *                      title 引导页标题
             *                      tips 引导页右上角按钮文字
             *                      showBack 引导页返回键是否显示
             */
            setupFaceUnlock: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.url)) {
                        throw "url can not be empty";
                    }
                    if (!isDefined(param.motion) || param.motion.length == 0) {
                        throw "motions can not be empty";
                    }
                    if (!isDefined(param.complexity)) {
                        param.complexity = "hell";
                    }
                    if (!isDefined(param.soundNotice)) {
                        param.soundNotice = true;
                    }
                    if (!isDefined(param.style)) {
                        param.style = 'default';
                    }
                    if (!isDefined(param.showBack)) {
                        param.showBack = false;
                    }
                    js2native.exec("FaceRecognizePlugin", "setupFaceUnlock", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            },
            /**
             * @method cancelFaceUnlock
             * 取消刷脸登录
             * @param 无
             */
            cancelFaceUnlock: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    js2native.exec("FaceRecognizePlugin", "cancelFaceUnlock", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.TouchPlugin
         */
        TouchPlugin: {
            /**
             * @method set
             * 设置当前账号的指纹解锁
             * @param style 默认值default，影响显示的样式
             */
            set: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    var style = !isDefined(param.style) ? "default" : param.style;
                    js2native.exec("TouchPlugin", "set", [style], this);
                }
            },

            /**
             * @method cancel
             * 取消当前用户的指纹解锁
             */
            cancel: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("TouchPlugin", "cancel", [], this);
                }
            },
            /**
             * @method functionSupport
             * 返回指纹解锁的状态

             */
            functionSupport: function(param) {
                if (typeof(param) === "object") {
                    _bindCallback(this, param.success, param.error);
                    js2native.exec("TouchPlugin", "functionSupport", [param.userID], this);
                }
            }
        },

        /**
         * @class plugins.LocalAuthPlugin
         */
        LocalAuthPlugin: {
            /**
             * @method auth:
             * 调取本地认证页面
             * @param authMode 字符串标识从左到右分别表示 刷脸，指纹，手势，如“101”表示用户开启刷脸和手势两种认证方式。
             */
            auth: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.authMode)) {
                        throw "authMode can not be empty";
                    }
                    js2native.exec("LocalAuthPlugin", "auth", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.DialogPlugin
         */
        DialogPlugin: {
            /**
             * @method show:
             * 调取本地警告或确认对话框
             * @param title 对话框的标题
             *      content 对话框的内容
             *      confirmBtnText 对话框的确认按钮中的内容
             *      cancelBtnText   对话框的取消内容，判断是否含有该字段，来判断出对话框的具体类型。若没有，则为警告对话框，若有，则为确认对话框
             */
            show: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.title)) {
                        throw "title can not be empty";
                    } else if (!isDefined(param.content)) {
                        throw "content can not be empty";
                    } else if (!isDefined(param.confirmBtnText)) {
                        throw "confirmBtnText can not be empty";
                    }
                    js2native.exec("DialogPlugin", "show", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.DiagnosePlugin
         */
        DiagnosePlugin: {

            /**
             * @method DiagnosePlugin.diagnose
             * 上传诊断信息文件
             */
            diagnose: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    js2native.exec("DiagnosePlugin", "diagnose", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.VideoRecordingPlugin
         */
        VideoRecordingPlugin: {
            /**
             * @method record:
             * 朗读范本文字并且录像
             * @param modelText 范本文字
             */
            record: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.modelText)) {
                        throw "modelText can not be empty";
                    }
                    if (!isDefined(param.uploadUrl)) {
                        throw "uploadUrl can not be empty";
                    }
                    js2native.exec("VideoRecordingPlugin", "record", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
         /**
         * @class plugins.AdDialogPlugin
         */
        AdDialogPlugin:{
            /**
             * @method show:
             * 显示广告
             * @param popAngle 弹出角度,有up_to_down、down_to_up、left_to_right、right_to_left、upleft_to_center、upright_to_center、downleft_to_center、downright_to_center八个值
             *        speed 弹性动画速度参数，有fast、normal、slow三种值
             *        bounciness 弹性动画弹性参数，有big、normal、small三种值
             *        widthRatio 宽占屏幕比，0-1之间的浮点型数据
             *        heightRatio 高占屏幕比，0-1之间的浮点型数据
             *        closeUrl 关闭按钮跳转url
             *        animType 动画弹出方式，有depth、rotate、zoom三种
             *        adInfoList 广告列表，这是一个AdInfo的列表，AdInfo的属性如下：
             *                   adType 广告类型，有h5页面和图片两种，值分别为H5、IMAGE
             *                   imgUrl 广告类型为图片时的图片url
             *                   imgCallbackUrl 广告类型为图片时的点击跳转url
             *                   h5Content 广告类型为h5页面时的h5代码
             */
            show: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.popAngle)) {
                        throw "popAngle can not be empty";
                    }
                    if (!isDefined(param.adInfoList)) {
                        throw "adInfoList can not be empty";
                    }
                    js2native.exec("AdDialogPlugin", "show", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.SearchPlugin
         */
        SearchPlugin:{
            /**
             * @method setSearchText:
             * 设置搜索内容
             * @param searchText 搜索内容
             */
            setSearchText: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    if (!isDefined(param.searchText)) {
                        throw "searchText can not be empty";
                    }
                    js2native.exec("SearchPlugin", "setSearchText", [param.searchText], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        },
        /**
         * @class plugins.HealthPlugin
         */
        HealthPlugin:{
            /**
             * @method getSteps:
             * 获取每日步数
             */
            getSteps: function(param) {
                _bindCallback(this, param.success, param.error);
                try {
                    js2native.exec("HealthPlugin", "getSteps", [JSON.stringify(param)], this);
                } catch (e) {
                    if (!isDefined(param.error)) {
                        param.error(e);
                    }
                }
            }
        }
    }


    /**
     * @class plugins.RESTPlugin
     * for android
     */
    plugins.RESTPlugin = {
        get: function(param) {
            _bindCallback(this, param.success, param.error);
            sendRESTRequest(param, "GET", this);
        },
        post: function(param) {
            _bindCallback(this, param.success, param.error);
            sendRESTRequest(param, "POST", this);
        },
        delete: function(param) {
            _bindCallback(this, param.success, param.error);
            sendRESTRequest(param, "DELETE", this);
        },
        put: function(param) {
            _bindCallback(this, param.success, param.error);
            sendRESTRequest(param, "PUT", this);
        }
    }

    /**
     * 由于安卓webView加载encodeUrl内容会自动decodeUrl。导致前端需要两次encodeUrl才能正确decodeUrl
     * 所以使用以下base64方法进行转换。
     */
    plugins.Base64Plugin = {
        // private property
        _keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

        // public method for encoding
        encode : function (input) {
            var output = "";
            var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
            var i = 0;
            input = this._utf8_encode(input);
            while (i < input.length) {
                chr1 = input.charCodeAt(i++);
                chr2 = input.charCodeAt(i++);
                chr3 = input.charCodeAt(i++);
                enc1 = chr1 >> 2;
                enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
                enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
                enc4 = chr3 & 63;
                if (isNaN(chr2)) {
                    enc3 = enc4 = 64;
                } else if (isNaN(chr3)) {
                    enc4 = 64;
                }
                output = output +
                    this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
                    this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);
            }
            return output;
        },

        // public method for decoding
        decode : function (input) {
            var output = "";
            var chr1, chr2, chr3;
            var enc1, enc2, enc3, enc4;
            var i = 0;
            input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
            while (i < input.length) {
                enc1 = this._keyStr.indexOf(input.charAt(i++));
                enc2 = this._keyStr.indexOf(input.charAt(i++));
                enc3 = this._keyStr.indexOf(input.charAt(i++));
                enc4 = this._keyStr.indexOf(input.charAt(i++));
                chr1 = (enc1 << 2) | (enc2 >> 4);
                chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
                chr3 = ((enc3 & 3) << 6) | enc4;
                output = output + String.fromCharCode(chr1);
                if (enc3 != 64) {
                    output = output + String.fromCharCode(chr2);
                }
                if (enc4 != 64) {
                    output = output + String.fromCharCode(chr3);
                }
            }
            output = this._utf8_decode(output);
            return output;
        },

        // private method for UTF-8 encoding
        _utf8_encode : function (string) {
            string = string.replace(/\r\n/g,"\n");
            var utftext = "";
            for (var n = 0; n < string.length; n++) {
                var c = string.charCodeAt(n);
                if (c < 128) {
                    utftext += String.fromCharCode(c);
                } else if((c > 127) && (c < 2048)) {
                    utftext += String.fromCharCode((c >> 6) | 192);
                    utftext += String.fromCharCode((c & 63) | 128);
                } else {
                    utftext += String.fromCharCode((c >> 12) | 224);
                    utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                    utftext += String.fromCharCode((c & 63) | 128);
                }

            }
            return utftext;
        },

        // private method for UTF-8 decoding
        _utf8_decode : function (utftext) {
            var string = "";
            var i = 0;
            var c,c1,c2,c3;
            c = c1 = c2 = 0;
            while ( i < utftext.length ) {
                c = utftext.charCodeAt(i);
                if (c < 128) {
                    string += String.fromCharCode(c);
                    i++;
                } else if((c > 191) && (c < 224)) {
                    c2 = utftext.charCodeAt(i+1);
                    string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                    i += 2;
                } else {
                    c2 = utftext.charCodeAt(i+1);
                    c3 = utftext.charCodeAt(i+2);
                    string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                    i += 3;
                }
            }
            return string;
        }
    }

    //旧接口兼容性代码
    /**
     * @member plugins.KeyBoardPlugin
     * @method inputPwdForVerify
     * 已过期方法
     * @deprecated 请使用 KeyBoardPlugin.show
     */
    plugins.KeyBoardPlugin.inputPwdForVerify = plugins.KeyBoardPlugin.show;
    /**
     * @member plugins.NumberKeyBoardPlugin
     * @method inputPwdForVerify
     * 已过期方法
     * @deprecated 请使用 NumberKeyBoardPlugin.show
     */
    plugins.NumberKeyBoardPlugin.inputPwdForVerify = plugins.NumberKeyBoardPlugin.show;
    /**
     * @member plugins.UrlProcessPlugin
     * @method popToPage
     * 已过期方法
     * @deprecated 请使用 UrlProcessPlugin.popToLevel
     */
    plugins.UrlProcessPlugin.popToPage = plugins.UrlProcessPlugin.popToLevel;


    var sendRESTRequest = function(param, method, invoker) {
        if (!isDefined(param.url)) {
            if (isDefined(param.error)) {
                param.error("url can not be empty");
            }
            return;
        }
        var requestData = !isDefined(param.requestData) ? {} : param.requestData;
        var headers = !isDefined(param.headers) ? {} : param.headers;
        var timeout = !isDefined(param.timeout) ? "30" : param.timeout;
        js2native.exec("RESTPlugin", method.toLowerCase(), [param.url, JSON.stringify(requestData), JSON.stringify(headers), timeout], invoker);
    }

    var _assignCallback = function(pluginObj) {
        for (var i in pluginObj) {
            for (var j in pluginObj[i]) {
                assignUtils.assign(pluginObj[i][j], {
                    success: function() {},
                    failure: function() {},
                    action: function(param) {
                        console.error("action接口将过期！");
                        this.call({}, param);
                    }
                });
            }
        }
    };

    _assignCallback(plugins);

    window.plugins = plugins;


})();